#!/usr/bin/python
# -*- coding: UTF-8 -*-

from urllib.request import urlopen 
import json
import websocket
import _thread
import grpc
import sys
from code_engine_client import bot_api_conn_grpc_pb2
from code_engine_client import bot_api_conn_grpc_pb2_grpc
import sys
import importlib
from concurrent.futures import ThreadPoolExecutor



class CodeEngineClient:
  bot_config_url = "https://apps-1254429489.cos.ap-beijing.myqcloud.com/enable_bot/enable_bot_config.json"
  event_ws_url = "ws://jimugaoshou.com:50510/code"
  cmd_svr_url = "jimugaoshou.com:50501"

  def __init__(self, openid, id):
    self.openid = openid
    self.id = id
    self.bot_config_map = {}
    self.drive_config_map = {}
    self.cmd_client = None
    self.event_client = None
    self.threads = None

  # outter functions
  def init(self):
    self.load_bot_config()
    self.load_drive_config()

  def run(self):
    self.init_net()
    self.init_threads()
    self.event_client.run_forever()

  def cmd_motor(self, bot_index, tag, x):
    print("cmd_motor")
    msg = self.format_cmd_motor_msg(bot_index, tag, x)
    self.send_cmd_msg(msg)

  def cmd_led(self, bot_index, tag, x):
    print("cmd_led")
    msg = self.format_cmd_led_msg(bot_index, tag, x)
    self.send_cmd_msg(msg)

  # end outter functions


  # inner functions
  def init_net(self):
    self.cmd_client = self.init_cmd_client()
    self.event_client =  self.init_event_client()

  # share cmd client, and not init event client, just for code engine svr
  def attach_net(self, cmd_client):
    self.cmd_client = cmd_client

  # thread pool
  def init_threads(self):
    self.threads = ThreadPoolExecutor(max_workers=10)

  def load_bot_config(self):
    self.bot_config_map = self.get_json_from_url(CodeEngineClient.bot_config_url)

  def load_drive_config(self):
    url = "https://imgs-1254429489.cos.ap-beijing.myqcloud.com/drive/"+self.openid+"/"+self.id+".json"
    self.drive_config_map = self.get_json_from_url(url)

  def get_json_from_url(self, url):
    html = urlopen(url)
    data = html.read()
    return json.loads(data)

  def init_cmd_client(self):
    print("init_cmd_client")
    channel = grpc.insecure_channel(CodeEngineClient.cmd_svr_url)
    return bot_api_conn_grpc_pb2_grpc.BotApiConnGrpcStub(channel)

  def init_event_client(self):
    websocket.enableTrace(True)
    return websocket.WebSocketApp(CodeEngineClient.event_ws_url,
                            on_open=self.event_on_open,
                            on_message=self.event_on_message,
                            on_error=self.event_on_error,
                            on_close=self.event_on_close)

  def event_on_open(self, ws):
    print("event_on_open")
    self.send_event_init_msg(ws)

  def event_on_message(self, ws, message):
    print("event_on_message")
    msg = json.loads(message)

    if msg["cmd"] == "event":
      self.event_loop(msg)

  def event_on_error(self, ws, error):
    print("event_on_error")

  def event_on_close(self, ws, close_status_code, close_msg):
    print("event_on_close")

  def event_loop(self, msg):
    print("event_loop")

    if msg["cmd"] == "event":
      self.process_event(msg)
    elif msg["cmd"] == "sensor":
      self.process_sensor(msg)

  def process_event(self, msg):
    module_name = self.get_local_module_name()
    function_name, param_cnt = self.get_seq_function_name(msg["seq"])

    if function_name != "":
      fun_path = module_name+"."+function_name
      self.threads.submit(self.dynamic_run, fun_path, param_cnt, msg["x"], msg["y"],'','','','')
    else:
      print("unknown msg")


  def process_sensor(self, msg):
    module_name = self.get_local_module_name()
    function_name, param_cnt = self.get_sensor_function_name(msg["model"])

    if function_name != "":
      fun_path = module_name+"."+function_name

      if msg["model"] == "line":
        self.threads.submit(self.dynamic_run, fun_path, param_cnt, msg["result"]["exist"], msg["result"]["offset"],'','','','')
      elif msg["model"] == "color":
        self.threads.submit(self.dynamic_run, fun_path, param_cnt, msg["result"]["r"], msg["result"]["g"],msg["result"]["b"],'','','')
      elif msg["model"] == "qrcode":
        self.threads.submit(self.dynamic_run, fun_path, param_cnt, msg["result"]["url"], '','','','','')
      elif msg["model"] == "tflite":
        self.threads.submit(self.dynamic_run, fun_path, param_cnt, msg["result"]["label"], msg["result"]["confidence"],msg["result"]["x"],msg["result"]["y"],msg["result"]["w"],msg["result"]["h"])

    else:
      print("unknown msg")


  def get_local_module_name(self):
    filename = sys.argv[0].split('/')[-1][0:-3]
    return filename

  def get_seq_function_name(self, seq):

    param_cnt = 1
    function_name = ""
    for unit in self.drive_config_map["units"]:
      if unit['seq'] == seq:
        if unit['ui']['type'] == "joystick":
          param_cnt = 2

        function_name = "on_event_"+unit['ui']['type']+"_"+str(seq)

    return function_name, param_cnt

  def get_sensor_function_name(self, model):
    param_cnt = 1
    function_name = ""

    if model == "line":
      param_cnt = 2
      function_name = "on_sensor_line"
    elif model == "color":
      param_cnt = 3
      function_name = "on_sensor_color"
    elif model == "tflite":
      param_cnt = 6
      function_name = "on_sensor_tflite"
    elif model == "qrcode":
      param_cnt = 1
      function_name = "on_sensor_qrcode"

    return function_name, param_cnt


  def reload_module(self, module_path):

    tmp = importlib.util.find_spec(module_path)
    if tmp == None:
        return None

    tmp = importlib.import_module(module_path)
    importlib.reload(tmp)

    return tmp

  def dynamic_run(self, fun_path, param_cnt, x1, x2, x3, x4, x5, x6):

    # try:
        module_path = fun_path.split('.')[0]
        fun_name = fun_path.split('.')[1]

        mod = self.reload_module(module_path)

        if hasattr(mod, fun_name):
          if param_cnt == 1:
            eval("mod."+fun_name)(self, x1)
          elif param_cnt == 2:
            eval("mod."+fun_name)(self, x1, x2)
          elif param_cnt == 3:
            eval("mod."+fun_name)(self, x1, x2, x3)
          elif param_cnt == 4:
            eval("mod."+fun_name)(self, x1, x2, x3, x4)
          elif param_cnt == 5:
            eval("mod."+fun_name)(self, x1, x2, x3, x4, x5)
          elif param_cnt == 6:
            eval("mod."+fun_name)(self, x1, x2, x3, x4, x5, x6)
        else:
            print("Error, no such function : "+fun_path)

    # except:
        # print("Error dynamic_run, fun_path : "+fun_path+", params : "+str(x1))


  def send_event_init_msg(self, ws):
    msg = {"openid": self.openid, "id": self.id, "cmd":"init"}
    data = json.dumps(msg)
    ws.send(data)


  def format_cmd_motor_msg(self, bot_index, tag, x):

    a1, a2 = self.get_pins_of_tag(tag)

    # format cmd
    msg = {
        "app_version":"1.0.0",
        "app_os": "api",
        "openid": self.openid,
        "cmd": "base",
        "bot_id": self.get_bot_id_of_index(bot_index),
        "a1": a1,
        "a2": a2,
        "speed": x
    }

    return msg

  def format_cmd_led_msg(self, bot_index, tag, x):
    # format cmd
    a1, a2 = self.get_pins_of_tag(tag)

    # format cmd
    msg = {
        "app_version":"1.0.0",
        "app_os": "api",
        "openid": self.openid,
        "cmd": "base",
        "bot_id": self.get_bot_id_of_index(bot_index),
        "a1": a1,
        "a2": a2,
        "speed": x
    }

    return msg

  def get_pins_of_tag(self, tag):
    return self.bot_config_map['control_bot']['base'][tag+'1'], self.bot_config_map['control_bot']['base'][tag+'2']

  def get_bot_id_of_index(self, bot_index):
    bots = self.drive_config_map["bots"]
    if bot_index < len(bots):
      return bots[bot_index]
    else:
      return ""

  def send_cmd_msg(self, msg):
    # format req
    req = bot_api_conn_grpc_pb2.SendBotMsgRequest(openid = msg["openid"], bot_id = msg["bot_id"], cmd = msg["cmd"], msg = json.dumps(msg))

    # send msg
    resp = self.cmd_client.SendBotMsg(req)


  # end inner functions



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from code_engine_client import bot_api_conn_grpc_pb2 as bot__api__conn__grpc__pb2


class BotApiConnGrpcStub(object):
    """service definition.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SendBotMsg = channel.unary_unary(
                '/bot_api_conn_grpc.BotApiConnGrpc/SendBotMsg',
                request_serializer=bot__api__conn__grpc__pb2.SendBotMsgRequest.SerializeToString,
                response_deserializer=bot__api__conn__grpc__pb2.SendBotMsgResponse.FromString,
                )
        self.DriveEventMsg = channel.unary_unary(
                '/bot_api_conn_grpc.BotApiConnGrpc/DriveEventMsg',
                request_serializer=bot__api__conn__grpc__pb2.DriveEventMsgRequest.SerializeToString,
                response_deserializer=bot__api__conn__grpc__pb2.DriveEventMsgResponse.FromString,
                )
        self.CodeSaveMsg = channel.unary_unary(
                '/bot_api_conn_grpc.BotApiConnGrpc/CodeSaveMsg',
                request_serializer=bot__api__conn__grpc__pb2.CodeSaveMsgRequest.SerializeToString,
                response_deserializer=bot__api__conn__grpc__pb2.CodeSaveMsgResponse.FromString,
                )
        self.CodeTestMsg = channel.unary_unary(
                '/bot_api_conn_grpc.BotApiConnGrpc/CodeTestMsg',
                request_serializer=bot__api__conn__grpc__pb2.CodeTestMsgRequest.SerializeToString,
                response_deserializer=bot__api__conn__grpc__pb2.CodeTestMsgResponse.FromString,
                )
        self.CodePcMsg = channel.unary_unary(
                '/bot_api_conn_grpc.BotApiConnGrpc/CodePcMsg',
                request_serializer=bot__api__conn__grpc__pb2.CodePcMsgRequest.SerializeToString,
                response_deserializer=bot__api__conn__grpc__pb2.CodePcMsgResponse.FromString,
                )


class BotApiConnGrpcServicer(object):
    """service definition.
    """

    def SendBotMsg(self, request, context):
        """send msg to bot
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DriveEventMsg(self, request, context):
        """event msg from drive
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CodeSaveMsg(self, request, context):
        """code save button
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CodeTestMsg(self, request, context):
        """code test button
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CodePcMsg(self, request, context):
        """code pc button
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_BotApiConnGrpcServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'SendBotMsg': grpc.unary_unary_rpc_method_handler(
                    servicer.SendBotMsg,
                    request_deserializer=bot__api__conn__grpc__pb2.SendBotMsgRequest.FromString,
                    response_serializer=bot__api__conn__grpc__pb2.SendBotMsgResponse.SerializeToString,
            ),
            'DriveEventMsg': grpc.unary_unary_rpc_method_handler(
                    servicer.DriveEventMsg,
                    request_deserializer=bot__api__conn__grpc__pb2.DriveEventMsgRequest.FromString,
                    response_serializer=bot__api__conn__grpc__pb2.DriveEventMsgResponse.SerializeToString,
            ),
            'CodeSaveMsg': grpc.unary_unary_rpc_method_handler(
                    servicer.CodeSaveMsg,
                    request_deserializer=bot__api__conn__grpc__pb2.CodeSaveMsgRequest.FromString,
                    response_serializer=bot__api__conn__grpc__pb2.CodeSaveMsgResponse.SerializeToString,
            ),
            'CodeTestMsg': grpc.unary_unary_rpc_method_handler(
                    servicer.CodeTestMsg,
                    request_deserializer=bot__api__conn__grpc__pb2.CodeTestMsgRequest.FromString,
                    response_serializer=bot__api__conn__grpc__pb2.CodeTestMsgResponse.SerializeToString,
            ),
            'CodePcMsg': grpc.unary_unary_rpc_method_handler(
                    servicer.CodePcMsg,
                    request_deserializer=bot__api__conn__grpc__pb2.CodePcMsgRequest.FromString,
                    response_serializer=bot__api__conn__grpc__pb2.CodePcMsgResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'bot_api_conn_grpc.BotApiConnGrpc', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class BotApiConnGrpc(object):
    """service definition.
    """

    @staticmethod
    def SendBotMsg(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/bot_api_conn_grpc.BotApiConnGrpc/SendBotMsg',
            bot__api__conn__grpc__pb2.SendBotMsgRequest.SerializeToString,
            bot__api__conn__grpc__pb2.SendBotMsgResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DriveEventMsg(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/bot_api_conn_grpc.BotApiConnGrpc/DriveEventMsg',
            bot__api__conn__grpc__pb2.DriveEventMsgRequest.SerializeToString,
            bot__api__conn__grpc__pb2.DriveEventMsgResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CodeSaveMsg(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/bot_api_conn_grpc.BotApiConnGrpc/CodeSaveMsg',
            bot__api__conn__grpc__pb2.CodeSaveMsgRequest.SerializeToString,
            bot__api__conn__grpc__pb2.CodeSaveMsgResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CodeTestMsg(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/bot_api_conn_grpc.BotApiConnGrpc/CodeTestMsg',
            bot__api__conn__grpc__pb2.CodeTestMsgRequest.SerializeToString,
            bot__api__conn__grpc__pb2.CodeTestMsgResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CodePcMsg(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/bot_api_conn_grpc.BotApiConnGrpc/CodePcMsg',
            bot__api__conn__grpc__pb2.CodePcMsgRequest.SerializeToString,
            bot__api__conn__grpc__pb2.CodePcMsgResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: bot_api_conn_grpc.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='bot_api_conn_grpc.proto',
  package='bot_api_conn_grpc',
  syntax='proto3',
  serialized_options=b'\n\034io.grpc.example.bot_api_connB\021bot_api_conn_grpcP\001\242\002\003HLW',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x17\x62ot_api_conn_grpc.proto\x12\x11\x62ot_api_conn_grpc\"M\n\x11SendBotMsgRequest\x12\x0e\n\x06openid\x18\x01 \x01(\t\x12\x0e\n\x06\x62ot_id\x18\x02 \x01(\t\x12\x0b\n\x03\x63md\x18\x03 \x01(\t\x12\x0b\n\x03msg\x18\x04 \x01(\t\"N\n\x12SendBotMsgResponse\x12\x0e\n\x06openid\x18\x01 \x01(\t\x12\x0e\n\x06\x62ot_id\x18\x02 \x01(\t\x12\x0b\n\x03\x63md\x18\x03 \x01(\t\x12\x0b\n\x03ret\x18\x04 \x01(\x05\"?\n\x14\x44riveEventMsgRequest\x12\x0e\n\x06openid\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x0b\n\x03msg\x18\x03 \x01(\t\"@\n\x15\x44riveEventMsgResponse\x12\x0e\n\x06openid\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x0b\n\x03ret\x18\x03 \x01(\x05\"0\n\x12\x43odeSaveMsgRequest\x12\x0e\n\x06openid\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\">\n\x13\x43odeSaveMsgResponse\x12\x0e\n\x06openid\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x0b\n\x03ret\x18\x03 \x01(\x05\"0\n\x12\x43odeTestMsgRequest\x12\x0e\n\x06openid\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\">\n\x13\x43odeTestMsgResponse\x12\x0e\n\x06openid\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x0b\n\x03ret\x18\x03 \x01(\x05\".\n\x10\x43odePcMsgRequest\x12\x0e\n\x06openid\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\"<\n\x11\x43odePcMsgResponse\x12\x0e\n\x06openid\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12\x0b\n\x03ret\x18\x03 \x01(\x05\x32\xed\x03\n\x0e\x42otApiConnGrpc\x12[\n\nSendBotMsg\x12$.bot_api_conn_grpc.SendBotMsgRequest\x1a%.bot_api_conn_grpc.SendBotMsgResponse\"\x00\x12\x64\n\rDriveEventMsg\x12\'.bot_api_conn_grpc.DriveEventMsgRequest\x1a(.bot_api_conn_grpc.DriveEventMsgResponse\"\x00\x12^\n\x0b\x43odeSaveMsg\x12%.bot_api_conn_grpc.CodeSaveMsgRequest\x1a&.bot_api_conn_grpc.CodeSaveMsgResponse\"\x00\x12^\n\x0b\x43odeTestMsg\x12%.bot_api_conn_grpc.CodeTestMsgRequest\x1a&.bot_api_conn_grpc.CodeTestMsgResponse\"\x00\x12X\n\tCodePcMsg\x12#.bot_api_conn_grpc.CodePcMsgRequest\x1a$.bot_api_conn_grpc.CodePcMsgResponse\"\x00\x42\x39\n\x1cio.grpc.example.bot_api_connB\x11\x62ot_api_conn_grpcP\x01\xa2\x02\x03HLWb\x06proto3'
)




_SENDBOTMSGREQUEST = _descriptor.Descriptor(
  name='SendBotMsgRequest',
  full_name='bot_api_conn_grpc.SendBotMsgRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='openid', full_name='bot_api_conn_grpc.SendBotMsgRequest.openid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bot_id', full_name='bot_api_conn_grpc.SendBotMsgRequest.bot_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cmd', full_name='bot_api_conn_grpc.SendBotMsgRequest.cmd', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg', full_name='bot_api_conn_grpc.SendBotMsgRequest.msg', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=123,
)


_SENDBOTMSGRESPONSE = _descriptor.Descriptor(
  name='SendBotMsgResponse',
  full_name='bot_api_conn_grpc.SendBotMsgResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='openid', full_name='bot_api_conn_grpc.SendBotMsgResponse.openid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bot_id', full_name='bot_api_conn_grpc.SendBotMsgResponse.bot_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cmd', full_name='bot_api_conn_grpc.SendBotMsgResponse.cmd', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ret', full_name='bot_api_conn_grpc.SendBotMsgResponse.ret', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=125,
  serialized_end=203,
)


_DRIVEEVENTMSGREQUEST = _descriptor.Descriptor(
  name='DriveEventMsgRequest',
  full_name='bot_api_conn_grpc.DriveEventMsgRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='openid', full_name='bot_api_conn_grpc.DriveEventMsgRequest.openid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='bot_api_conn_grpc.DriveEventMsgRequest.id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='msg', full_name='bot_api_conn_grpc.DriveEventMsgRequest.msg', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=205,
  serialized_end=268,
)


_DRIVEEVENTMSGRESPONSE = _descriptor.Descriptor(
  name='DriveEventMsgResponse',
  full_name='bot_api_conn_grpc.DriveEventMsgResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='openid', full_name='bot_api_conn_grpc.DriveEventMsgResponse.openid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='bot_api_conn_grpc.DriveEventMsgResponse.id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ret', full_name='bot_api_conn_grpc.DriveEventMsgResponse.ret', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=270,
  serialized_end=334,
)


_CODESAVEMSGREQUEST = _descriptor.Descriptor(
  name='CodeSaveMsgRequest',
  full_name='bot_api_conn_grpc.CodeSaveMsgRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='openid', full_name='bot_api_conn_grpc.CodeSaveMsgRequest.openid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='bot_api_conn_grpc.CodeSaveMsgRequest.id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=336,
  serialized_end=384,
)


_CODESAVEMSGRESPONSE = _descriptor.Descriptor(
  name='CodeSaveMsgResponse',
  full_name='bot_api_conn_grpc.CodeSaveMsgResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='openid', full_name='bot_api_conn_grpc.CodeSaveMsgResponse.openid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='bot_api_conn_grpc.CodeSaveMsgResponse.id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ret', full_name='bot_api_conn_grpc.CodeSaveMsgResponse.ret', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=386,
  serialized_end=448,
)


_CODETESTMSGREQUEST = _descriptor.Descriptor(
  name='CodeTestMsgRequest',
  full_name='bot_api_conn_grpc.CodeTestMsgRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='openid', full_name='bot_api_conn_grpc.CodeTestMsgRequest.openid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='bot_api_conn_grpc.CodeTestMsgRequest.id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=450,
  serialized_end=498,
)


_CODETESTMSGRESPONSE = _descriptor.Descriptor(
  name='CodeTestMsgResponse',
  full_name='bot_api_conn_grpc.CodeTestMsgResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='openid', full_name='bot_api_conn_grpc.CodeTestMsgResponse.openid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='bot_api_conn_grpc.CodeTestMsgResponse.id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ret', full_name='bot_api_conn_grpc.CodeTestMsgResponse.ret', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=500,
  serialized_end=562,
)


_CODEPCMSGREQUEST = _descriptor.Descriptor(
  name='CodePcMsgRequest',
  full_name='bot_api_conn_grpc.CodePcMsgRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='openid', full_name='bot_api_conn_grpc.CodePcMsgRequest.openid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='bot_api_conn_grpc.CodePcMsgRequest.id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=564,
  serialized_end=610,
)


_CODEPCMSGRESPONSE = _descriptor.Descriptor(
  name='CodePcMsgResponse',
  full_name='bot_api_conn_grpc.CodePcMsgResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='openid', full_name='bot_api_conn_grpc.CodePcMsgResponse.openid', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='bot_api_conn_grpc.CodePcMsgResponse.id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ret', full_name='bot_api_conn_grpc.CodePcMsgResponse.ret', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=612,
  serialized_end=672,
)

DESCRIPTOR.message_types_by_name['SendBotMsgRequest'] = _SENDBOTMSGREQUEST
DESCRIPTOR.message_types_by_name['SendBotMsgResponse'] = _SENDBOTMSGRESPONSE
DESCRIPTOR.message_types_by_name['DriveEventMsgRequest'] = _DRIVEEVENTMSGREQUEST
DESCRIPTOR.message_types_by_name['DriveEventMsgResponse'] = _DRIVEEVENTMSGRESPONSE
DESCRIPTOR.message_types_by_name['CodeSaveMsgRequest'] = _CODESAVEMSGREQUEST
DESCRIPTOR.message_types_by_name['CodeSaveMsgResponse'] = _CODESAVEMSGRESPONSE
DESCRIPTOR.message_types_by_name['CodeTestMsgRequest'] = _CODETESTMSGREQUEST
DESCRIPTOR.message_types_by_name['CodeTestMsgResponse'] = _CODETESTMSGRESPONSE
DESCRIPTOR.message_types_by_name['CodePcMsgRequest'] = _CODEPCMSGREQUEST
DESCRIPTOR.message_types_by_name['CodePcMsgResponse'] = _CODEPCMSGRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SendBotMsgRequest = _reflection.GeneratedProtocolMessageType('SendBotMsgRequest', (_message.Message,), {
  'DESCRIPTOR' : _SENDBOTMSGREQUEST,
  '__module__' : 'bot_api_conn_grpc_pb2'
  # @@protoc_insertion_point(class_scope:bot_api_conn_grpc.SendBotMsgRequest)
  })
_sym_db.RegisterMessage(SendBotMsgRequest)

SendBotMsgResponse = _reflection.GeneratedProtocolMessageType('SendBotMsgResponse', (_message.Message,), {
  'DESCRIPTOR' : _SENDBOTMSGRESPONSE,
  '__module__' : 'bot_api_conn_grpc_pb2'
  # @@protoc_insertion_point(class_scope:bot_api_conn_grpc.SendBotMsgResponse)
  })
_sym_db.RegisterMessage(SendBotMsgResponse)

DriveEventMsgRequest = _reflection.GeneratedProtocolMessageType('DriveEventMsgRequest', (_message.Message,), {
  'DESCRIPTOR' : _DRIVEEVENTMSGREQUEST,
  '__module__' : 'bot_api_conn_grpc_pb2'
  # @@protoc_insertion_point(class_scope:bot_api_conn_grpc.DriveEventMsgRequest)
  })
_sym_db.RegisterMessage(DriveEventMsgRequest)

DriveEventMsgResponse = _reflection.GeneratedProtocolMessageType('DriveEventMsgResponse', (_message.Message,), {
  'DESCRIPTOR' : _DRIVEEVENTMSGRESPONSE,
  '__module__' : 'bot_api_conn_grpc_pb2'
  # @@protoc_insertion_point(class_scope:bot_api_conn_grpc.DriveEventMsgResponse)
  })
_sym_db.RegisterMessage(DriveEventMsgResponse)

CodeSaveMsgRequest = _reflection.GeneratedProtocolMessageType('CodeSaveMsgRequest', (_message.Message,), {
  'DESCRIPTOR' : _CODESAVEMSGREQUEST,
  '__module__' : 'bot_api_conn_grpc_pb2'
  # @@protoc_insertion_point(class_scope:bot_api_conn_grpc.CodeSaveMsgRequest)
  })
_sym_db.RegisterMessage(CodeSaveMsgRequest)

CodeSaveMsgResponse = _reflection.GeneratedProtocolMessageType('CodeSaveMsgResponse', (_message.Message,), {
  'DESCRIPTOR' : _CODESAVEMSGRESPONSE,
  '__module__' : 'bot_api_conn_grpc_pb2'
  # @@protoc_insertion_point(class_scope:bot_api_conn_grpc.CodeSaveMsgResponse)
  })
_sym_db.RegisterMessage(CodeSaveMsgResponse)

CodeTestMsgRequest = _reflection.GeneratedProtocolMessageType('CodeTestMsgRequest', (_message.Message,), {
  'DESCRIPTOR' : _CODETESTMSGREQUEST,
  '__module__' : 'bot_api_conn_grpc_pb2'
  # @@protoc_insertion_point(class_scope:bot_api_conn_grpc.CodeTestMsgRequest)
  })
_sym_db.RegisterMessage(CodeTestMsgRequest)

CodeTestMsgResponse = _reflection.GeneratedProtocolMessageType('CodeTestMsgResponse', (_message.Message,), {
  'DESCRIPTOR' : _CODETESTMSGRESPONSE,
  '__module__' : 'bot_api_conn_grpc_pb2'
  # @@protoc_insertion_point(class_scope:bot_api_conn_grpc.CodeTestMsgResponse)
  })
_sym_db.RegisterMessage(CodeTestMsgResponse)

CodePcMsgRequest = _reflection.GeneratedProtocolMessageType('CodePcMsgRequest', (_message.Message,), {
  'DESCRIPTOR' : _CODEPCMSGREQUEST,
  '__module__' : 'bot_api_conn_grpc_pb2'
  # @@protoc_insertion_point(class_scope:bot_api_conn_grpc.CodePcMsgRequest)
  })
_sym_db.RegisterMessage(CodePcMsgRequest)

CodePcMsgResponse = _reflection.GeneratedProtocolMessageType('CodePcMsgResponse', (_message.Message,), {
  'DESCRIPTOR' : _CODEPCMSGRESPONSE,
  '__module__' : 'bot_api_conn_grpc_pb2'
  # @@protoc_insertion_point(class_scope:bot_api_conn_grpc.CodePcMsgResponse)
  })
_sym_db.RegisterMessage(CodePcMsgResponse)


DESCRIPTOR._options = None

_BOTAPICONNGRPC = _descriptor.ServiceDescriptor(
  name='BotApiConnGrpc',
  full_name='bot_api_conn_grpc.BotApiConnGrpc',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=675,
  serialized_end=1168,
  methods=[
  _descriptor.MethodDescriptor(
    name='SendBotMsg',
    full_name='bot_api_conn_grpc.BotApiConnGrpc.SendBotMsg',
    index=0,
    containing_service=None,
    input_type=_SENDBOTMSGREQUEST,
    output_type=_SENDBOTMSGRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DriveEventMsg',
    full_name='bot_api_conn_grpc.BotApiConnGrpc.DriveEventMsg',
    index=1,
    containing_service=None,
    input_type=_DRIVEEVENTMSGREQUEST,
    output_type=_DRIVEEVENTMSGRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CodeSaveMsg',
    full_name='bot_api_conn_grpc.BotApiConnGrpc.CodeSaveMsg',
    index=2,
    containing_service=None,
    input_type=_CODESAVEMSGREQUEST,
    output_type=_CODESAVEMSGRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CodeTestMsg',
    full_name='bot_api_conn_grpc.BotApiConnGrpc.CodeTestMsg',
    index=3,
    containing_service=None,
    input_type=_CODETESTMSGREQUEST,
    output_type=_CODETESTMSGRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CodePcMsg',
    full_name='bot_api_conn_grpc.BotApiConnGrpc.CodePcMsg',
    index=4,
    containing_service=None,
    input_type=_CODEPCMSGREQUEST,
    output_type=_CODEPCMSGRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_BOTAPICONNGRPC)

DESCRIPTOR.services_by_name['BotApiConnGrpc'] = _BOTAPICONNGRPC

# @@protoc_insertion_point(module_scope)
